"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const zlib_1 = require("zlib");
const querystring_1 = require("querystring");
const getRawBody = require("raw-body");
const createError = require("http-errors");
// Allowed whitespace is defined in RFC 7159.
// http://www.rfc-editor.org/rfc/rfc7159.txt
const STRICT_JSON_REGEXP = /^[\x20\x09\x0a\x0d]*[\[\{]/;
exports.DEFAULT_LIMIT = 100 * 1024; // 100kb.
exports.DEFAULT_ENCODING = 'utf8';
/**
 * Supported encodings.
 */
exports.decoders = {
    deflate: (stream) => stream.pipe(zlib_1.createInflate()),
    gzip: (stream) => stream.pipe(zlib_1.createGunzip())
};
/**
 * Default JSON parser.
 */
function jsonParse(str) {
    if (str.length === 0)
        return {};
    if (!STRICT_JSON_REGEXP.test(str)) {
        throw createError(400, 'Invalid JSON, only supports object and array');
    }
    return JSON.parse(str);
}
/**
 * Default support content types.
 */
exports.jsonTypes = ['application/json'];
exports.formTypes = ['application/x-www-form-urlencoded'];
exports.textTypes = ['text/plain', 'text/html'];
/**
 * Parse any incoming stream.
 */
function parse(stream, headers, options = {}) {
    const type = getType(headers);
    if (!type)
        return Promise.reject(createError(415, 'Missing content-type'));
    const jsonType = options.jsonTypes || exports.jsonTypes;
    const formType = options.formTypes || exports.formTypes;
    const textType = options.textTypes || exports.textTypes;
    if (jsonType.indexOf(type) > -1)
        return json(stream, headers, options);
    if (formType.indexOf(type) > -1)
        return form(stream, headers, options);
    if (textType.indexOf(type) > -1)
        return text(stream, headers, options);
    return Promise.reject(createError(415, `Unsupported content-type: ${type}`));
}
exports.parse = parse;
/**
 * Parse the stream as JSON.
 */
function json(stream, headers, options = {}) {
    return text(stream, headers, options).then(options.jsonParse || jsonParse);
}
exports.json = json;
/**
 * Parse the stream as a form.
 */
function form(stream, headers, options = {}) {
    return text(stream, headers, options).then(options.formParse || querystring_1.parse);
}
exports.form = form;
/**
 * Parse the stream as text.
 */
function text(stream, headers, options = {}) {
    const { error, body, length } = inflate(stream, headers, options.decoders);
    if (error)
        return Promise.reject(error);
    const limit = options.limit || exports.DEFAULT_LIMIT;
    const encoding = options.encoding || exports.DEFAULT_ENCODING;
    return getRawBody(body, { length, limit, encoding });
}
exports.text = text;
/**
 * Inflate the readable stream.
 */
function inflate(stream, headers, encoding = exports.decoders) {
    const enc = String(headers['content-encoding'] || 'identity').toLowerCase();
    if (enc === 'identity') {
        return {
            body: stream,
            length: ~~headers['content-length']
        };
    }
    if (encoding[enc]) {
        return {
            body: encoding[enc](stream)
        };
    }
    return {
        body: stream,
        error: createError(`Unsupported content-encoding: ${enc}`)
    };
}
exports.inflate = inflate;
/**
 * Return the `content-type` string.
 */
function getType(headers) {
    const contentType = headers['content-type'];
    if (!contentType)
        return undefined;
    const type = Array.isArray(contentType) ? contentType[0] : contentType;
    const index = type.indexOf(';');
    return index > -1 ? type.substr(0, index) : type;
}
exports.getType = getType;
//# sourceMappingURL=index.js.map